@@ -67,6 +67,7 @@ import os
 import re
 import sys
 import io
+import logging
 
 # xml format
 import lxml.etree as ET
@@ -76,7 +77,7 @@ import html2text
 
 
 # version
-__version__         = '0.7.0'
+__version__         = '0.7.1'
 
 # BUILTIN ICONS
 ICON_EXCLAMATION    = 'yes'
@@ -88,6 +89,13 @@ ICON_PRIO1          = 'full-1'
 ICON_PRIO2          = 'full-2'
 
 
+# logging
+logging.basicConfig(
+        format='%(name)s - %(levelname)-8s - %(message)s',
+        level=logging.WARNING,
+        )
+
+
 # MINDMAP
 
 class Mindmap(object):
@@ -110,7 +118,32 @@ class Mindmap(object):
     _global_node_id_seed = datetime.datetime.now().strftime('%y%m%d')
 
 
-    def __init__(self, path='', mtype='freeplane', version='1.3.0', id=''):
+    def __init__(self,
+            path='',
+            mtype='freeplane',
+            version='1.3.0',
+            id='',
+            log_level="warning",
+            ):
+
+
+
+
+        #
+        # adjust logging level to user's wishes
+        #
+
+        if log_level.lower() == "debug":
+            logging.getLogger().setLevel(logging.DEBUG)
+        elif log_level.lower() == "info":
+            logging.getLogger().setLevel(logging.INFO)
+        elif log_level.lower() == "warning":
+            logging.getLogger().setLevel(logging.WARNING)
+        elif log_level.lower() == "error":
+            logging.getLogger().setLevel(logging.ERROR)
+        else:
+            logging.getLogger().setLevel(logging.WARNING)
+            logging.warning("log level mismatch in user arguments. setting to WARNING.")
 
 
 
@@ -151,7 +184,7 @@ class Mindmap(object):
             # check if command is provided in script
             if not hasattr(self, args.command):
 
-                print( 'Unrecognized command' )
+                logging.error('Unrecognized command. EXITING.')
                 parser.print_help()
                 sys.exit(1)
 
@@ -212,7 +245,7 @@ class Mindmap(object):
                 with io.open(self._path, "r", encoding="utf-8") as fpMap:
                     strFirstLine = fpMap.readline()
             except:
-                print("[ WARNING: format mismatch in mindmap file vw. UTF-8 ]")
+                logging.warning("format mismatch in mindmap file vs. UTF-8")
                 retry = True
 
             # in case there are wrong encodings when trying to read as UTF-8,
@@ -221,9 +254,11 @@ class Mindmap(object):
             # workaround
 
             if retry:
-                with io.open(self._path, "r", encoding="windows-1252") as fpMap:
-                    strFirstLine = fpMap.readline()
-
+                try:
+                    with io.open(self._path, "r", encoding="windows-1252") as fpMap:
+                        strFirstLine = fpMap.readline()
+                except:
+                    logging.warning("format mismatch in mindmap file vs. windows-1252")
 
             # now, analyze the characters in the first line of the mindmap file
             # and try to find the "freeplane" token which will contain the
@@ -260,7 +295,46 @@ class Mindmap(object):
             # read entire mindmap and evaluate structure
             #
 
-            self._mindmap = ET.parse(self._path, parser=xmlparser)
+            # some Freeplane versions produce invalid XML syntax when writing
+            # the mindmap into file. here, these invalid syntaxes are to be
+            # removed from the file, before using and parsing the file.
+
+            try:
+                self._mindmap = ET.parse(self._path, parser=xmlparser)
+
+            except ET.XMLSyntaxError:
+                logging.warning("invalid XML syntax. will try to fix it temporarily...")
+
+                # write sanitized file into temporary file
+                _basename = "_" + os.path.basename(self._path)
+                _dirname  = os.path.dirname(self._path)
+
+                # ensure temp file is not yet present
+                while os.path.isfile(os.path.join(_dirname, _basename)):
+                    _basename = "_" + _basename
+                _temp_file = os.path.join(_dirname, _basename)
+
+                # read original XML file
+                with io.open(self._path, "r", encoding="utf-8") as _file:
+                    _content = _file.read()
+
+                # sanitize content
+                _content = _content.replace("&nbsp;", "&#160;")
+
+                # create and write temp file
+                with io.open(_temp_file, "w", encoding="utf-8") as _file:
+                    _file.write(_content)
+
+                # repeat open of mindmap
+                self._mindmap = ET.parse(_temp_file, parser=xmlparser)
+
+                # remove temporary file
+                os.remove(_temp_file)
+
+                logging.info("... XML source was successfully sanitized.")
+
+            # now that the XML file has been read in in a valid way, the normal
+            # XML parsing is to take place within the module's functionalities.
 
             # get root of mindmap
             self._root = self._mindmap.getroot()
@@ -574,7 +648,7 @@ class Mindmap(object):
 
         # style
         if style:
-            print("[ WARNING: style attribute not implemented, yet ]")
+            logging.warning("style attribute not implemented, yet")
 
         return node
 
